﻿
## AutomationML Editor Plugin development

### Version 6.4

**AutomationML Editor** supports the integration of Plugins.

With the introduction of version 6.0, only plug-ins that support at least .NET 5.0 are supported. Plugins that have been developed for an older .NET version can no longer be integrated by this version. 

The development tools, used so far for plug-in development, have been extended by additional tools for Plugin development for editor version 6.0. If you are a Plugin developer, you can choose the appropriate development platform. V5 developers use the [V5 platform](https://github.com/AutomationML/AMLEditorPlugin/tree/master/PlugInDevelopment_V5). V6 developers use the new [V6 platform](https://github.com/AutomationML/AMLEditorPlugin/tree/master/PlugInDevelopment_V6). The V6 platform also offers new Plugin examples.

V6 Plugin Development can use a new [AutomationML Editor API Package](https://www.nuget.org/packages/Aml.Editor.API).


> **Plugins are licensed by their respective owners. AutomationML assumes no responsibility for third-party Plugins 
> or grants licenses for third-party Plugins.**

To be able to implement and deploy Plugins the following resources are provided:

1. A contract, defining interfaces which are importable by the Editor. The DLL is available at https://www.nuget.org/packages/Aml.Editor.Plugin.Contract/.

2. Detailed documentation, example Plugins and a Sandbox project supporting tests and deployment, available at https://github.com/AutomationML/AMLEditorPlugin/. Please note, that the sandbox project no longer supports plugin development using MEF but the .NET core assembly load context as the underlying technology.

3. A Toolkit, which nicely integrates with the AutomationML editor for viewing of AutomationML structures, available at https://www.nuget.org/packages/Aml.Toolkit/.

Until editor version 6.3 Plugin development and installation was based on the MEF Framework https://docs.microsoft.com/en-us/dotnet/framework/mef/. Since editor version 6.4, plugins are no longer loaded using MEF means, but via the API of the Assembly Load Context https://learn.microsoft.com/en-us/dotnet/core/dependency-loading/understanding-assemblyloadcontext. Plugins that use MEF can no longer be loaded by the editor from version 6.4.0. All plugins published on NuGet have been converted accordingly.

Best way to implement a Plugin is to start with the Sandbox project and one of the contained examples, to get an understanding how Plugins are developed. Own Plugin developments should start with a copy of one of the provided examples. Modify the code and test the deployment within the Sandbox. The Sandbox mimics the docking and loading behaviour of the AutomationML Editor. If you can load and run your Plugin in the Sandbox, you should be able to run it in your installed AutomationML Editor - please ensure, that there are no name conflicts with already existing Plugins. Since **version 5.1.3** of the AutomationML editor, an integrated Plugin Manager can be used to install a Plugin. 
Each Plugin should be provided in the **`Aml.Editor.Plugin`** namespace and provide an 'About box'. Only those Plugins may be published to other users. The AutomationML Editors Plugin Manager will list all Plugins, which are published on NuGet and are correctly named using the  **`Aml.Editor.Plugin`** namespace prefix.
